using System;
using System.Collections.Generic;
using System.Text;
using System.Timers;

namespace WeakRef
{
  class Program
  {
    static void Main(string[] args)
    {
        // Objektinstanz erzeugen
        // mit einer "strong reference"
        ClassA objA1 = new ClassA();

        // Erzeugung einer "short weak reference" fr das Objekt
        // Angabe von "true" fr eine "long weak reference"
        WeakReference wr = new WeakReference(objA1, true);

        // Objekt fr die weak reference setzen
        wr.Target = objA1;

        // Strong reference lschen
        objA1 = null;

        if (objA1 == null)
        {
          Console.WriteLine("objA1 auf null gesetzt: objA1 ist nun null.");
        }

        // Timer erzeugen und initialisieren
        Timer aTimer = new Timer();
        aTimer.Interval = 5000;
        aTimer.Enabled = true;

        // Zhler initialisieren
        int count = 1;

        // Anonyme Methode fr Timer-Tick
        aTimer.Elapsed += delegate(object sender, ElapsedEventArgs e)
        {
          if (objA1 == null)
          {
            Console.WriteLine("objA1 ist null");

            // Garbage collection ausfhren
            if (count == 5)
            {
              // Ggf. eine strong reference neu erstellen
              //objA1 = (ClassA)wr.Target;
              GC.Collect();
            }

            if (count == 8)
            {
              // Dieser Code wird nur mit einer
              // "long weak reference" erreicht
              objA1 = (ClassA)wr.Target;

            }

            if (wr.IsAlive == false)
            {
              // Objekt wurde weggerumt und
              // alle Referenzen sind anulliert
              Console.WriteLine("Objekt weggerumt");
              aTimer.Stop();
            }
            else
            {
              Console.WriteLine("Count: {0}, Objekt lebt noch. Long weak reference: {1}",
                  count.ToString(), wr.TrackResurrection.ToString());

              count++;
            }
          }
          else
          {
            // Eine strong reference ist vorhanden
            // Das Objekt kann nicht weggerumt werden
            Console.WriteLine("Strong reference vorhanden");
            aTimer.Stop();
          }
        };

        Console.ReadLine();
    }
  }

  public class ClassA
  {
    public ClassA()
    {
    }

    ~ClassA()
    {
      Console.WriteLine("Finalized.");
    }

    void MakeSomeGarbage()
    {
      // Objekte erzeugen und freigeben
      // um den Speicher mit unntzen Objekten zu fllen
      Version vt;

      for (int i = 0; i < 10000; i++)
      {
        vt = new Version();
      }
    }
  }
}



// -------- Code example output --------
//
// A short weak reference creates
// the following output:
// ------------------------------------
// After setting objA1 to null: objA1 is null.
// objA1 is null
// count 1, still alive. long weak reference: False
// objA1 is null
// count 2, still alive. long weak reference: False
// objA1 is null
// count 3, still alive. long weak reference: False
// objA1 is null
// count 4, still alive. long weak reference: False
// objA1 is null
// Object collected
// Finalized.
// ------------------------------------
//
//
// A long weak reference creates
// the following output, being
// ressurected at count 5.
// ------------------------------------
// After setting objA1 to null: objA1 is null.
// objA1 is null
// count 1, still alive. long weak reference: True
// objA1 is null
// count 2, still alive. long weak reference: True
// objA1 is null
// count 3, still alive. long weak reference: True
// objA1 is null
// count 4, still alive. long weak reference: True
// objA1 is null
// count 5, still alive. long weak reference: True
// Finalized.
// objA1 is null
// count 6, still alive. long weak reference: True
// objA1 is null
// count 7, still alive. long weak reference: True
// objA1 is null
// count 8, still alive. long weak reference: True
// Strong reference applied
// ----------------------------------
//
//
// A weak reference creates
// the following output with a
// strong reference applied
// at count 5.
// ----------------------------------
// After setting objA1 to null: objA1 is null.
// objA1 is null
// count 1, still alive. long weak reference: False
// objA1 is null
// count 2, still alive. long weak reference: False
// objA1 is null
// count 3, still alive. long weak reference: False
// objA1 is null
// count 4, still alive. long weak reference: False
// objA1 is null
// count 5, still alive. long weak reference: False
// Strong reference applied
// ------------------------------------
//